/* get all tables and views with an optional primary key name */

select rel.rdb$relation_name, rc.rdb$constraint_name as primarykey_name, rel.rdb$description, rel.rdb$external_file, rel.rdb$system_flag, rel.rdb$view_source, rel.rdb$owner_name
from rdb$relations rel
left outer join rdb$relation_constraints rc on (rc.rdb$relation_name = rel.rdb$relation_name and rc.rdb$constraint_type = 'PRIMARY KEY')
ORDER BY rel.rdb$relation_name ASC




/* get columns for a table or view and optional information about any domain information  */


select rf.rdb$field_name, f.rdb$field_type, f.rdb$field_sub_type, rf.rdb$description,
       rf.rdb$null_flag, rf.rdb$default_source as field_default, f.rdb$default_source as domain_default,
       f.rdb$field_scale, f.rdb$computed_source, rf.rdb$field_position,
       f.rdb$field_length, f.rdb$character_length, rf.rdb$field_source, rdb$segment_length,
       f.rdb$character_set_id, rf.rdb$collation_id as field_collation, f.rdb$collation_id as domain_collation,
       f.rdb$dimensions, fd.rdb$upper_bound, fd.rdb$lower_bound
, f.rdb$field_precision
from rdb$fields f left outer join rdb$field_dimensions fd on f.rdb$field_name = fd.rdb$field_name,
     rdb$relation_fields rf
where f.rdb$field_name = rf.rdb$field_source and
      rf.rdb$relation_name = 'YOURTABLE'
ORDER BY rf.rdb$field_position ASC
       , fd.rdb$dimension ASC



/* get user defined domains, including collation, defaults, check constraint etc */

select f.rdb$field_name, rdb$description,
       rdb$field_scale, rdb$field_type, rdb$field_sub_type,
       rdb$field_length, rdb$null_flag, rdb$character_set_id, rdb$collation_id, rdb$character_length,
       rdb$default_source, rdb$validation_source, rdb$segment_length, f.rdb$dimensions, fd.rdb$upper_bound, fd.rdb$lower_bound
, f.rdb$field_precision
from rdb$fields f left outer join rdb$field_dimensions fd on f.rdb$field_name = fd.rdb$field_name
where rdb$system_flag <> 1 and f.rdb$field_name not like 'RDB$%'
ORDER BY f.rdb$field_name ASC
       , fd.rdb$dimension ASC



/* get foreign key information and the target constraint they point to. Warning: on multi-column constraints,
this will return multiple rows per constraint, one for each column *

select rc.rdb$constraint_name, rc.rdb$constraint_type, rc.rdb$relation_name, rc.rdb$index_name, idx.rdb$index_inactive,     idxs.rdb$field_name as detail_field, ref.rdb$const_name_uq, ref.rdb$update_rule, ref.rdb$delete_rule 
from rdb$relation_constraints rc, rdb$ref_constraints ref, rdb$index_segments idxs, rdb$indices idx where rc.rdb$constraint_name = ref.rdb$constraint_name and idxs.rdb$index_name = rc.rdb$index_name and      idxs.rdb$index_name = idx.rdb$index_name
ORDER BY idxs.rdb$field_position ASC


/* get unique constraints, for multi-column constraints, the same constraint appears multiple times, once
for each column */

select rc.rdb$constraint_name, rc.rdb$relation_name, idx.rdb$field_name, idx.rdb$index_name  from rdb$relation_constraints rc, rdb$index_segments idx  where rc.rdb$constraint_type = 'UNIQUE' and rc.rdb$index_name = idx.rdb$index_name
ORDER BY rc.rdb$constraint_name ASC
       , rc.rdb$relation_name ASC
       , idx.rdb$field_position ASC


/* check constraints, two rows for each check constraint, different (system defined) trigger names */

select rc.rdb$constraint_name as constraint_name,
       rc.rdb$relation_name as relation_name,
       cc.rdb$trigger_name as trigger_name,
       tr.rdb$trigger_source
from rdb$relation_constraints rc,
     rdb$check_constraints cc,
     rdb$triggers tr
where rc.rdb$constraint_type = 'CHECK'
  and rc.rdb$constraint_name = cc.rdb$constraint_name
  and tr.rdb$trigger_name = cc.rdb$trigger_name
ORDER BY rc.rdb$constraint_name ASC




/* get indices with their fields, one row for each indexed field */

select idx.rdb$field_name, i.rdb$index_name, i.rdb$description, i.rdb$unique_flag, i.rdb$index_inactive, i.rdb$relation_name, i.rdb$index_type
from rdb$indices i left outer join rdb$relation_constraints rc on i.rdb$index_name = rc.rdb$index_name,
 rdb$index_segments idx
where idx.rdb$index_name = i.rdb$index_name and i.rdb$system_flag is null and rc.rdb$constraint_type is null
ORDER BY i.rdb$index_name ASC
       , idx.rdb$field_position ASC




/* triggers */

select rdb$trigger_name, rdb$trigger_type, rdb$trigger_sequence, rdb$description,
       rdb$trigger_source, rdb$trigger_inactive, rdb$relation_name
from rdb$triggers
where (rdb$system_flag = 0 or rdb$system_flag is null)
  and rdb$trigger_name not in
     (select cc.rdb$trigger_name
        from rdb$check_constraints cc,
             rdb$relation_constraints rc
       where rc.rdb$constraint_type <> 'NOT NULL'
         and rc.rdb$constraint_name = cc.rdb$constraint_name)
 and rdb$trigger_name not like 'CHECK%'
ORDER BY rdb$trigger_name ASC




